/****************************************************************************************/
/*  MSGLOG.H                                                                            */
/*                                                                                      */
/*  Author:                                                                             */
/*  Description:                                                                        */
/*                                                                                      */
/*  The contents of this file are subject to the Jet3D Public License                   */
/*  Version 1.02 (the "License"); you may not use this file except in                   */
/*  compliance with the License. You may obtain a copy of the License at                */
/*  http://www.jet3d.com                                                                */
/*                                                                                      */
/*  Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/*  The Original Code is Jet3D, released December 12, 1999.                             */
/*  Copyright (C) 1996-1999 Eclipse Entertainment, L.L.C. All Rights Reserved           */
/*                                                                                      */
/****************************************************************************************/
#pragma once

#ifndef MSGLOG_H
#define MSGLOG_H

#ifdef __cplusplus
extern "C" {
#endif

	typedef struct tagMsgLog MsgLog ;

MsgLog * MsgLog_Create();
void MsgLog_Destroy( MsgLog **hMsgLog );
BOOL MsgLog_StartRecord( MsgLog * pMsgLog, char * Name );
BOOL MsgLog_Record( MsgLog * pMsgLog, MSG* pMsg, int32 WndSigniture );
void MsgLog_EndRecord( MsgLog * pMsgLog );
BOOL MsgLog_IsLogMsg( MSG* pMsg );
BOOL MsgLog_IsRecording(MsgLog * pMsgLog);
BOOL MsgLog_StartPlay( MsgLog * pMsgLog, char * Name );
void MsgLog_EndPlay(MsgLog * pMsgLog);
void MsgLog_Play( MsgLog * pMsgLog );
BOOL MsgLog_IsPlaying(MsgLog * pMsgLog);


#ifdef __cplusplus
}
#endif

#endif // Prevent multiple inclusion
/* EOF: MsgLog.h */